package cn.com.qmth.stmms.biz.mark.dao;

import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.persistence.LockModeType;

import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Lock;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;

import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.common.enums.LibraryStatus;

public interface MarkLibraryDao extends JpaRepository<MarkLibrary, Integer>, JpaSpecificationExecutor<MarkLibrary> {

    @Lock(LockModeType.PESSIMISTIC_WRITE)
    @Query("select l from MarkLibrary l where l.id=?1")
    MarkLibrary findByIdForLock(Integer id);

    List<MarkLibrary> findByExamIdAndSubjectCode(Integer examId, String subjectCode, Pageable page);

    List<MarkLibrary> findByExamIdAndSubjectCodeAndGroupNumberAndStatus(Integer examId, String subjectCode,
            Integer groupNumber, LibraryStatus status, Pageable page);

    List<MarkLibrary> findByExamIdAndSubjectCodeAndGroupNumberAndStatusIn(Integer examId, String subjectCode,
            Integer groupNumber, Set<LibraryStatus> statusSet, Pageable page);

    @Query("select l from MarkLibrary l where l.studentId=?1 order by l.groupNumber ")
    List<MarkLibrary> findByStudentId(Integer studentId);

    List<MarkLibrary> findByStudentIdAndGroupNumber(Integer studentId, Integer groupNumber);

    List<MarkLibrary> findByStudentIdAndGroupNumberAndStatus(Integer studentId, Integer groupNumber,
            LibraryStatus status);

    @Query("select count(*) from MarkLibrary f where f.studentId=?1 and f.groupNumber=?2")
    long countByStudentIdAndGroupNumber(Integer studentId, Integer groupNumber);

    @Query("select count(*) from MarkLibrary f where f.studentId=?1 and f.markerId=?2")
    long countByStudentIdAndMarkerId(Integer studentId, Integer markerId);

    @Query("select count(*) from MarkLibrary f where f.studentId=?1 and f.markerId=?2 and f.id!=?3")
    long countByStudentIdAndMarkerIdAndIdNotEqual(Integer studentId, Integer markerId, Integer id);

    @Query("select distinct l.campusId from MarkLibrary l where l.examId=?1 and l.tags is not null")
    List<Integer> findTagCampusId(Integer examId);

    @Query("select distinct l.subjectCode from MarkLibrary l where l.examId=?1 and l.tags is not null")
    List<String> findTagSubjectCode(Integer examId);

    @Modifying(clearAutomatically = true)
    @Query("update MarkLibrary m set m.status=?4, m.tags=null, m.markerId=null, m.markerTime=null, m.markerScore=null, m.markerScoreList=null , m.headerId=null , m.headerTime=null , m.headerScore=null , m.headerScoreList=null where m.examId=?1 and m.subjectCode=?2 and m.groupNumber=?3")
    void resetByExamIdAndSubjectCodeAndGroupNumber(Integer examId, String subjectCode, Integer groupNumber,
            LibraryStatus status);

    @Modifying(clearAutomatically = true)
    @Query("update MarkLibrary m set m.status=?2, m.tags=null, m.markerId=null, m.markerTime=null, m.markerScore=null, m.markerScoreList=null, m.headerId=null , m.headerTime=null , m.headerScore=null , m.headerScoreList=null  where m.markerId=?1 and m.status!=?3 and m.status!=?4 ")
    void resetByMarkerId(Integer markerId, LibraryStatus status,LibraryStatus libraryStatus1,LibraryStatus libraryStatus2);

    @Modifying(clearAutomatically = true)
    @Query("update MarkLibrary m set m.status=?2, m.tags=null, m.markerId=null, m.markerTime=null, m.markerScore=null, m.markerScoreList=null , m.headerId=null , m.headerTime=null , m.headerScore=null , m.headerScoreList=null where m.id=?1")
    void resetById(Integer id, LibraryStatus status);

    @Query("select f.markerId, count(*) as markerCount from MarkLibrary f where f.examId=?1 and f.status=?2 group by f.markerId")
    List<Object[]> countByMarkerAndStatus(Integer examId, LibraryStatus status);

    @Query("select f.markerId, count(*) as markerCount from MarkLibrary f where f.examId=?1 and f.subjectCode=?2 and f.status=?3 group by f.markerId")
    List<Object[]> countByMarkerAndStatus(Integer examId, String subjectCode, LibraryStatus status);

    @Modifying
    @Query("delete MarkLibrary m where m.examId=?1 and m.subjectCode=?2 and m.groupNumber=?3")
    void deleteByExamIdAndSubjectCodeAndGroupNumber(Integer examId, String subjectCode, Integer groupNumber);

    @Modifying
    @Query("delete MarkLibrary m where m.studentId=?1")
    void deleteByStudentId(Integer studentId);

    @Query("select l from MarkLibrary l where l.studentId=?1 and l.status=?2 order by l.groupNumber ")
    List<MarkLibrary> findByStudentIdAndStatus(Integer studentId, LibraryStatus status);

    @Modifying(clearAutomatically = true)
    @Query("update MarkLibrary m set m.headerId=?3, m.headerScore=?4, m.headerScoreList=?5, m.headerTime=?6, m.status=?7 "
            + " where m.studentId=?1 and m.groupNumber=?2")
    void updateHeaderResult(Integer studentId, Integer groupNumber, Integer userId, Double totalScore, String scoreList,
            Date updateTime, LibraryStatus arbitrated);

    @Modifying(clearAutomatically = true)
    @Query("update MarkLibrary m set m.status=?3 where m.studentId=?1 and m.groupNumber=?2")
    void updateByStudentIdAndGroupNumber(Integer studentId, Integer groupNumber, LibraryStatus status);

    @Modifying(clearAutomatically = true)
    @Query("update MarkLibrary l set l.status=?2, l.markerId=?3, l.markerScore=?4, l.markerScoreList=?5, l.markerTime=?6, l.tags=?7 "
            + "where l.id=?1 and l.status in (?8) and (l.markerId=null or l.markerId=?3)")
    int updateMarkerResult(Integer id, LibraryStatus newStatus, Integer markerId, Double markerScore,
            String markerScoreList, Date markerTime, String tags, LibraryStatus... previousStatus);

}
